import logging
import re

from discord.ext import commands

from message import ErrorMessage


class ErrorHandler(commands.Cog):
  """A cog for global error handling."""

  def __init__(self, bot: commands.Bot, logger: logging.Logger):
    self.bot = bot
    self.logger = logger

  @commands.Cog.listener()
  async def on_command_error(self, ctx: commands.Context, error: commands.CommandError):
    command_name = ctx.command.name
    if isinstance(error, commands.CommandNotFound):
      self.logger.error(error)
      return
    if isinstance(error, commands.MissingPermissions):
      message = "You are missing the required permissions to run this command!"
    elif isinstance(error, commands.UserInputError):
      message = "Something about your input was wrong, please check your input and try again!"
    elif isinstance(error, commands.CommandError):
      message = str(error)
    else:
      message = "Oh no! Something went wrong while running the command!"

    ansi_escape = re.compile(r'\x1B(?:[@-Z\\-_]|\[[0-?]*[ -/]*[@-~])')
    message = ansi_escape.sub("", message)

    self.logger.error('Error during command "%s": %s', command_name, message)
    embed = ErrorMessage(message, command_name=command_name)
    await ctx.send(embed=embed)


def setup(bot: commands.Bot):
  bot.add_cog(ErrorHandler(bot))
