import re

import discord


class BotMessage(discord.Embed):
  def __init__(self, **kwargs):
    title = kwargs.pop("title", None)[:256]
    if title is not None:
      # Max embed title length is 256
      title = title[:256]
    super().__init__(
        **kwargs,
        title=title
    )


class ErrorMessage(BotMessage):
  def __init__(self, message, *, command_name: str = None):
    title = message
    if command_name:
      title = f'Error during command "{command_name}"'
    description = None
    if match := re.search(r"(?P<error>\w+Error): ?(ERROR: ?)?(?P<message>.*): ?Traceback", message):
      description = f"{match.group('error')}: {match.group('message')}"
    super().__init__(
        title=title,
        description=description,
        color=discord.Color.red()
    )


class NowPlayingMessage(BotMessage):
  def __init__(self, title, url):
    super().__init__(
        title=f"Now playing: {title}",
        description=url,
        color=discord.Color.green()
    )


class QueuedMessage(BotMessage):
  def __init__(self, title, url):
    super().__init__(
        title=f"Queued: {title}",
        description=url,
        color=discord.Color.blue()
    )
